/// <reference types="node" />
import events from 'events';
import { AxiosInstance, AxiosRequestConfig } from 'axios';
import { default as WebSocket } from 'ws';
/**
 * Main object used to talk to the Casambi Cloud API.
 */
export declare class CasambiAPI {
    apiKey: string;
    axios: AxiosInstance;
    /**
     * Represents the WebSocket connection to the Casambi Cloud API.
     * https://developer.casambi.com/#ws-service
     */
    connection: CasambiConnection;
    /**
     * You'll need an API key to use this class. See:
     * https://developer.casambi.com/#api-get-started
     * @param apiKey
     */
    constructor(apiKey: string);
    get(path: string, config?: AxiosRequestConfig): Promise<any>;
    post(path: string, data: any, config?: AxiosRequestConfig): Promise<any>;
    /**
     * Log into given Casambi Network and return a session object.
     * https://developer.casambi.com/#create-network-session
     * @param email
     * @param password
     */
    createNetworkSession(email: string, password: string): Promise<CasambiNetworkSession>;
    /**
     * Log into given user/site account and return a session object.
     * https://developer.casambi.com/#create-user-session
     * @param email
     * @param password
     */
    createUserSession(email: string, password: string): Promise<CasambiUserSession>;
    /**
     * https://developer.casambi.com/#request-fixture-information
     * @param fixtureId
     */
    requestFixtureInformation(fixtureId: number): Promise<any>;
    /**
     * https://developer.casambi.com/#request-fixture-icon
     * @param fixtureId
     */
    requestFixtureIcon(fixtureId: number): Promise<ArrayBuffer>;
}
/**
 * Represents a session for a specific Casambi Network.
 * Create using CasambiAPI.createNetworkSession().
 *
 * Events:
 * - "wireOpen"
 *   Wire has been opened.
 * - "wireClose"
 *   Wire has been closed.
 * - "unitChanged", peerChanged", "networkUpdated"
 *   Received a network/unit event.
 *   https://developer.casambi.com/#ws-method-types
 * - "wireStatus"
 *   Received a wire status message.
 *   https://developer.casambi.com/#ws-wire-status-types
 */
export declare class CasambiNetworkSession extends events.EventEmitter {
    api: CasambiAPI;
    networkInfo: any;
    sessionId: string;
    wireId: number;
    constructor(api: CasambiAPI, networkInfo: any, sessionId: string);
    get(path: string, config?: AxiosRequestConfig): Promise<any>;
    /**
     * https://developer.casambi.com/#request-network-information
     */
    requestInformation(): Promise<any>;
    /**
     * https://developer.casambi.com/#request-network-unit-list
     */
    requestUnitList(): Promise<any>;
    /**
     * https://developer.casambi.com/#request-network-unit-state
     * @param unitId
     */
    requestUnitState(unitId: number): Promise<any>;
    /**
     * https://developer.casambi.com/#request-network-state
     */
    requestState(): Promise<any>;
    /**
     * https://developer.casambi.com/#request-network-groups
     */
    requestGroups(): Promise<any>;
    /**
     * https://developer.casambi.com/#request-network-scenes
     */
    requestScenes(): Promise<any>;
    /**
     * https://developer.casambi.com/#request-network-datapoints
     * @param filterOptions
     */
    requestDatapoints(filterOptions: string): Promise<any>;
    /**
     * https://developer.casambi.com/#request-network-unit-icon
     * @param unitId
     */
    requestUnitIcon(unitId: number): Promise<ArrayBuffer>;
    /**
     * https://developer.casambi.com/#request-network-gallery
     */
    requestGallery(): Promise<any>;
    /**
     * https://developer.casambi.com/#request-network-image
     * @param imageId
     */
    requestImage(imageId: string): Promise<ArrayBuffer>;
    /**
     * Open connection wire for this network.
     * Needs to be called to start monitoring network events.
     */
    openWire(): Promise<void>;
    /**
     * Close connection wire for this network.
     */
    closeWire(): Promise<void>;
    /**
     * Control a Casambi unit (turn light on/off, etc.) on this network.
     * Opens a wire first, if needed.
     * https://developer.casambi.com/#ws-control-messages
     * @param unitId
     * @param targetControls
     */
    sendControlUnit(unitId: number, targetControls: any): Promise<void>;
    private onConnectionClose;
    private onMessage;
}
/**
 * Represents the result of a user login.
 * Returned by CasambiAPI.createUserSession().
 * https://developer.casambi.com/#create-user-session
 */
export declare class CasambiUserSession {
    api: CasambiAPI;
    sites: any;
    networks: any;
    sessionId: string;
    constructor(api: CasambiAPI, sites: any, networks: any, sessionId: string);
    /**
     * Returns an array of CasambiSite objects for user's sites.
     */
    createSites(): CasambiSite[];
    /**
     * Create network session objects for all networks of this user.
     */
    createNetworkSessions(): CasambiNetworkSession[];
}
/**
 * Represents a single site belonging to a user.
 * Returned by CasambiUserSession.createSites().
 */
export declare class CasambiSite {
    session: CasambiUserSession;
    siteInfo: any;
    constructor(session: CasambiUserSession, siteInfo: any);
    /**
     * Create network session objects for all networks of this site.
     */
    createNetworkSessions(): CasambiNetworkSession[];
}
/**
 * Represents a WebSocket connection to the Casambi Cloud API.
 * https://developer.casambi.com/#ws-service
 *
 * Events:
 * - "open"
 *   Connection to the server established.
 * - "close"
 *   Connection lost.
 * - "timeout"
 *   Connection timed out. Followed by "close".
 * - "message"
 *   Message received from the server.
 */
export declare class CasambiConnection extends events.EventEmitter {
    apiKey: string;
    ws?: WebSocket;
    private nextWireId;
    private pingInterval?;
    private pongTimeout?;
    constructor(apiKey: string);
    /**
     * Connect the WebSocket to the server.
     * Must be followed immediately by openWire().
     */
    open(): Promise<void>;
    /**
     * Close the WebSocket connection to the server.
     */
    close(): void;
    sendPing(): void;
    newWireId(): number;
    /**
     * Open wire for a Casambi network session.
     * Connects to the server first, if needed.
     * Promise resolves with the newly assigned wire ID.
     * https://developer.casambi.com/#ws-open-message
     * @param networkId
     * @param sessionId
     */
    openWire(networkId: string, sessionId: string): Promise<number>;
    /**
     * Close/pause a wire.
     * https://developer.casambi.com/#ws-close-message
     * @param wireId
     */
    closeWire(wireId: number): Promise<void>;
    /**
     * Control a Casambi unit (turn light on/off, etc.).
     * https://developer.casambi.com/#ws-control-messages
     * @param wireId
     * @param unitId
     * @param targetControls
     */
    sendControlUnit(wireId: number, unitId: number, targetControls: any): Promise<void>;
    /**
     * Send raw json to the server.
     * @param message
     */
    sendMessage(message: any): Promise<void>;
    private onOpen;
    private onClose;
    private onPong;
    private onMessage;
}
//# sourceMappingURL=casambi.d.ts.map