"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.LuminaireAccessory = void 0;
// When sliding the brightness bar in the Home App, multiple unit state updates are received
// from the cloud. To avoid interfering with what the user is doing, the updates are passed
// to HomeKit only after a delay. If more updates are received within that time, previous
// updates are dropped and the delay is extended.
const UNITCHANGED_DELAY = 500;
/**
 * Platform Accessory
 * An instance of this class is created for each accessory your platform registers
 * Each accessory may expose multiple services of different service types.
 */
class LuminaireAccessory {
    constructor(platform, accessory, session, unitInfo) {
        this.platform = platform;
        this.accessory = accessory;
        this.session = session;
        this.unitName = unitInfo.name;
        this.unitId = unitInfo.id;
        this.minCCT = 2700;
        this.maxCCT = 4000;
        this.brightness = 0.0;
        const fixtureInfo = accessory.context.fixtureInfo;
        this.platform.log.debug('Fixture info for unit', unitInfo.name, fixtureInfo);
        // set accessory information
        this.accessory.getService(this.platform.Service.AccessoryInformation)
            .setCharacteristic(this.platform.Characteristic.Manufacturer, fixtureInfo.vendor)
            .setCharacteristic(this.platform.Characteristic.Model, fixtureInfo.model)
            .setCharacteristic(this.platform.Characteristic.SerialNumber, unitInfo.address);
        // get the LightBulb service if it exists, otherwise create a new LightBulb service
        // you can create multiple services for each accessory
        this.service = this.accessory.getService(this.platform.Service.Lightbulb) || this.accessory.addService(this.platform.Service.Lightbulb);
        // To avoid "Cannot add a Service with the same UUID another Service without also defining a unique 'subtype' property." error,
        // when creating multiple services of the same type, you need to use the following syntax to specify a name and subtype id:
        // this.accessory.getService('NAME') ?? this.accessory.addService(this.platform.Service.Lightbulb, 'NAME', 'USER_DEFINED_SUBTYPE');
        // set the service name, this is what is displayed as the default name on the Home app
        this.service.setCharacteristic(this.platform.Characteristic.Name, unitInfo.name);
        // each service must implement at-minimum the "required characteristics" for the given service type
        // see https://developers.homebridge.io/#/service/Lightbulb
        let hasOnCharacteristic = false;
        for (const controlInfo of fixtureInfo.controls) {
            switch (controlInfo.type) {
                case 'dimmer': {
                    // register handlers for the On/Off Characteristic
                    this.service.getCharacteristic(this.platform.Characteristic.On)
                        .on('set', this.setOn.bind(this));
                    hasOnCharacteristic = true;
                    // register handlers for the Brightness Characteristic
                    this.brightness = this.service.getCharacteristic(this.platform.Characteristic.Brightness)
                        .on('set', this.setBrightness.bind(this))
                        .value;
                    break;
                }
                case 'temperature': {
                    // register handlers for the ColorTemperature Characteristic
                    this.service.getCharacteristic(this.platform.Characteristic.ColorTemperature)
                        .on('set', this.setColorTemperature.bind(this));
                    break;
                }
                case 'vertical': {
                    // this controls the direction of the light (up/down)
                    // if disabled in config, make sure the separate Lightbulb service is removed
                    if (this.platform.config.verticalControl !== 'separate') {
                        this.platform.log.info('Control type vertical is disabled in config for unit', unitInfo.name);
                        const verticalService = this.accessory.getServiceById(this.platform.Service.Lightbulb, 'vertical');
                        if (verticalService) {
                            this.accessory.removeService(verticalService);
                        }
                        break;
                    }
                    // Since there is no corresponding characteristic for this in HomeKit yet, add a separate Lightbulb service
                    // and use its Brightness characteristic to control the direction.
                    this.verticalService = this.accessory.getServiceById(this.platform.Service.Lightbulb, 'vertical')
                        || this.accessory.addService(new this.platform.Service.Lightbulb('', 'vertical'));
                    // set the service name, this is what is displayed as the default name on the Home app
                    this.service.setCharacteristic(this.platform.Characteristic.Name, unitInfo.name + ' Brightness');
                    this.verticalService.setCharacteristic(this.platform.Characteristic.Name, unitInfo.name + ' Up Down');
                    // register handlers for the On/Off Characteristic
                    this.verticalService.getCharacteristic(this.platform.Characteristic.On)
                        .on('set', this.setOn.bind(this));
                    // register handlers for the Brightness Characteristic
                    this.verticalService.getCharacteristic(this.platform.Characteristic.Brightness)
                        .on('set', this.setVerticalBrightness.bind(this));
                    break;
                }
                default: {
                    this.platform.log.info('Unsupported control type', controlInfo.type, 'for unit', unitInfo.name);
                    break;
                }
            }
        }
        if (!hasOnCharacteristic) {
            this.platform.log.error('Luminaire', unitInfo.name, 'has no On characteristic');
        }
        // monitor for state changes; current values are always sent after connecting
        session.on('unitChanged', this.onUnitChanged.bind(this));
    }
    /**
     * Handle "SET" requests from HomeKit
     * These are sent when the user changes the state of an accessory, for example, turning on a Light bulb.
     */
    setOn(value, callback) {
        this.platform.log.debug('Set characteristic On of unit', this.unitName, 'to', value);
        this.sendControlUnit(callback, {
            Dimmer: {
                value: (value ? this.brightness || 100 : 0) / 100.0,
            },
        });
    }
    /**
     * Handle "SET" requests from HomeKit
     * These are sent when the user changes the state of an accessory, for example, changing the Brightness
     */
    setBrightness(value, callback) {
        this.platform.log.debug('Set characteristic Brightness of unit', this.unitName, 'to', value);
        this.brightness = value;
        this.sendControlUnit(callback, {
            Dimmer: {
                value: this.brightness / 100.0,
            },
        });
    }
    /**
     * Handle "SET" requests from HomeKit
     * These are sent when the user changes the state of an accessory, for example, changing the Brightness
     */
    setVerticalBrightness(value, callback) {
        this.platform.log.debug('Set characteristic vertical/Brightness of unit', this.unitName, 'to', value);
        this.sendControlUnit(callback, {
            Vertical: {
                value: 1.0 - value / 100.0,
            },
        });
    }
    /**
     * Handle "SET" requests from HomeKit
     * These are sent when the user changes the state of an accessory, for example, changing the Color Temperature
     */
    setColorTemperature(value, callback) {
        this.platform.log.debug('Set characteristic ColorTemperature of unit', this.unitName, 'to', value);
        this.sendControlUnit(callback, {
            ColorTemperature: {
                value: Math.min(Math.max(1e6 / value, this.minCCT), this.maxCCT),
            },
            Colorsource: {
                source: 'TW',
            },
        });
    }
    /**
     * Handle notifications from Casambi Cloud.
     */
    onUnitChanged(message) {
        // filter out notifications for this accessory
        if (message.id === this.unitId) {
            this.platform.log.debug('Received unitChanged for', this.unitName, message);
            // update characteristics after a delay
            if (this.unitChangedTimeout) {
                clearTimeout(this.unitChangedTimeout);
            }
            this.unitChangedTimeout = setTimeout(() => {
                this.updateCharacteristicsFromUnitChanged(message);
                this.unitChangedTimeout = undefined;
            }, UNITCHANGED_DELAY);
        }
    }
    updateCharacteristicsFromUnitChanged(message) {
        this.platform.log.debug('Updating characteristics from unitChanged for', this.unitName);
        for (const controlInfo of message.controls) {
            switch (controlInfo.type) {
                case 'Dimmer': {
                    const brightness = controlInfo.value * 100.0;
                    this.service.updateCharacteristic(this.platform.Characteristic.On, brightness > 0);
                    if (brightness > 0) {
                        this.service.updateCharacteristic(this.platform.Characteristic.Brightness, brightness);
                        this.brightness = brightness;
                    }
                    break;
                }
                case 'Vertical': {
                    if (this.verticalService) {
                        const isOn = this.service.getCharacteristic(this.platform.Characteristic.On).value;
                        // mimic the on/off state of the main Lightbulb service
                        this.verticalService.updateCharacteristic(this.platform.Characteristic.On, isOn);
                        if (isOn) {
                            const position = (1.0 - controlInfo.value) * 100.0;
                            this.verticalService.updateCharacteristic(this.platform.Characteristic.Brightness, position);
                        }
                    }
                    break;
                }
                case 'CCT': {
                    // update CCT limits, used when sending color temperature
                    this.minCCT = controlInfo.min;
                    this.maxCCT = controlInfo.max;
                    const mired = 1e6 / controlInfo.value;
                    this.service.updateCharacteristic(this.platform.Characteristic.ColorTemperature, mired);
                    break;
                }
                default: {
                    this.platform.log.debug('Unsupported control type', controlInfo.type, 'for unit', this.unitName);
                    break;
                }
            }
        }
    }
    sendControlUnit(callback, targetControls) {
        this.platform.log.debug('Send controlUnit for', this.unitName, targetControls);
        this.session.sendControlUnit(this.unitId, targetControls)
            .then(() => callback(null))
            .catch(err => callback(err));
    }
}
exports.LuminaireAccessory = LuminaireAccessory;
//# sourceMappingURL=luminaire.js.map